@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Schedule::Activity::Attribute;
 
-our $VERSION='0.1.4';
+our $VERSION='0.1.5';
 
 sub new {
 	my ($ref,%opt)=@_;
@@ -35,7 +35,7 @@ sub register {
 sub log {
 	my ($self,$tm)=@_;
 	if(!defined($tm)) { return $self }
-	foreach my $A (values %{$$self{attr}}) { $A->log($tm) }
+	foreach my $A (values %{$$self{attr}}) { $A->change(tm=>$tm,_log=>1) }
 	return $self;
 }
 
